import React, { useState } from 'react'
import { Button, message, TreeSelect, Table, Popconfirm } from 'antd'
import type { FormInstance } from 'antd'
import ProForm, { ModalForm, ProFormText } from '@ant-design/pro-form'
import type { ProFormColumnsType } from '@ant-design/pro-form'
import { EditableProTable } from '@ant-design/pro-table'
import ShowTitleMenu from '../Attendance/components/ShowTitleMenu'
import { useModel, useRequest } from 'umi'
import {
  addBusinessgroup,
  changeBusinessGroupStatus,
  delBusinessGroupById,
  getBusinessgroupList,
  updateBusinessGroup
} from '@/services/user/api'
import type { ColumnsType } from 'antd/lib/table'
import { Delete } from '@icon-park/react'
import { useRef } from 'react'

enum modalType {
  CREATE = 0,
  UPDATE = 1
}

interface StageSettingsType {
  id: React.Key
  name?: string
  stage?: string
  percentage?: number
}

const defaultData = [
  {
    id: 1,
    stage: '结束',
    name: '赢单',
    percentage: '100'
  },
  {
    id: 2,
    stage: '结束',
    name: '赢单',
    percentage: '0'
  },
  {
    id: 3,
    stage: '结束',
    name: '无效',
    percentage: '0'
  }
]

const Contact: React.FC = () => {
  const { department } = useModel('department')
  const formRef = useRef<FormInstance>(null)

  const [editableKeys, setEditableRowKeys] = useState<React.Key[]>()

  const [state, setState] = useState({
    id: '',
    dataSource: [],
    visible: false,
    type: modalType.CREATE
  })

  const columns: ProFormColumnsType<StageSettingsType>[] = [
    {
      title: '阶段',
      dataIndex: 'stage',
      width: '15%'
    },
    {
      title: '阶段名称',
      dataIndex: 'name',
      width: '45%'
    },
    {
      title: '赢单率',
      dataIndex: 'percentage',
      width: '30%',
      valueType: 'text',
      fieldProps: {
        addonAfter: '%'
      }
    },
    {
      title: '操作',
      valueType: 'option',
      width: '10%'
    }
  ]
  const { run: tryGetList } = useRequest(getBusinessgroupList, {
    onSuccess: (result: API.BusinessgroupItem[]) => {
      setState({ ...state, dataSource: result })
    }
  })

  const { run: tryAddBusinessgroup } = useRequest(addBusinessgroup, {
    manual: true,
    onSuccess: async () => {
      tryGetList()
      message.success('添加成功')
    }
  })
  const { run: tryUpdateBusinessGroup } = useRequest(updateBusinessGroup, {
    manual: true,
    onSuccess: async () => {
      tryGetList()
      message.success('添加成功')
    }
  })

  const { run: tryDelBusinessGroup } = useRequest(delBusinessGroupById, {
    manual: true,
    onSuccess: async () => {
      await tryGetList()
    }
  })

  const { run: tryChangeBusinessGroupStatus } = useRequest(changeBusinessGroupStatus, {
    manual: true,
    onSuccess: async () => {
      await tryGetList()
    }
  })

  const mainColumns: ColumnsType<API.BusinessgroupItem> = [
    {
      dataIndex: 'name',
      title: '组名称'
    },
    {
      dataIndex: 'departmentName',
      title: '应用部门'
    },
    {
      dataIndex: 'createTime',
      title: '创建时间'
    },
    {
      dataIndex: 'createName',
      title: '创建人'
    },
    {
      dataIndex: 'activation',
      title: '状态',
      render: text => <span className={!text ? 'text-red-500' : ''}>{text ? '启用' : '停用'}</span>
    },
    {
      dataIndex: 'opreate',
      title: '操作',
      render: (_, record) => (
        <div className="divide-x divide-bg-gray-400 flex flex-row">
          <div
            className="pr-2 text-primary cursor-pointer hover:text-hex-967bbd"
            onClick={() => {
              formRef.current?.setFieldsValue({ ...record })
              setState({ ...state, visible: true, type: modalType.UPDATE })
            }}>
            编辑
          </div>
          {record.activation ? (
            <Popconfirm
              title="是否停用此商机组"
              onConfirm={() =>
                tryChangeBusinessGroupStatus({
                  id: record.id,
                  activation: Number(!record.activation)
                })
              }>
              <div className="px-2 text-red-500 cursor-pointer hover:text-red-600">停用</div>
            </Popconfirm>
          ) : null}
          {!record.activation && (
            <div
              className="px-2 text-primary cursor-pointer hover:text-hex-967bbd"
              onClick={() =>
                tryChangeBusinessGroupStatus({
                  id: record.id,
                  activation: Number(!record.activation)
                })
              }>
              启用
            </div>
          )}

          <Popconfirm title="确认删除？" onConfirm={() => tryDelBusinessGroup(record.id)}>
            <div className="pl-2 text-hex-fd3995 cursor-pointer hover:text-hex-e7026e">
              <Delete />
            </div>
          </Popconfirm>
        </div>
      )
    }
  ]
  return (
    <div className="h-full w-full flex flex-row">
      <ShowTitleMenu options={[{ label: '商机状态组', value: 1 }]} defaultValue={1} />
      <div className="w-max-3/4">
        <div className="ml-8 bg-white p-4 shadow-md shadow-hex-3e2c5a relative">
          <div className="pb-3 border-b border-b-black border-opacity-8 mb-3 flex items-center justify-between">
            <div>商机状态组</div>
            <Button
              type="primary"
              onClick={() => setState({ ...state, visible: true, type: modalType.CREATE })}>
              添加新组
            </Button>
          </div>
          <Table
            columns={mainColumns}
            dataSource={state.dataSource}
            bordered
            rowKey={row => row.id}
          />
        </div>
      </div>
      <ModalForm
        visible={state.visible}
        formRef={formRef}
        onVisibleChange={show => setState({ ...state, visible: show })}
        title={state.type === modalType.CREATE ? '创建商机角色' : '更新商机角色'}
        onFinish={async value => {
          if (state.type === modalType.CREATE) {
            await tryAddBusinessgroup({
              ...value,
              stageSettings: JSON.stringify(value.stageSettings)
            })
          } else {
            await tryUpdateBusinessGroup({ ...value })
          }
          return true
        }}>
        <ProFormText name="name" label="组名称" required />
        <ProFormText name="id" hidden />
        {/* <ProFormText name="department" label="应用部门" required /> */}
        <ProForm.Item name="departmentIds" label="应用部门" required>
          <TreeSelect treeData={department} multiple={true} />
        </ProForm.Item>
        <ProForm.Item
          label="阶段设置"
          name="stageSettings"
          initialValue={defaultData}
          trigger="onValuesChange">
          <EditableProTable<StageSettingsType>
            rowKey="id"
            toolBarRender={false}
            columns={columns}
            recordCreatorProps={{
              newRecordType: 'dataSource',
              position: 'top',
              record: index => ({
                id: Date.now(),
                stage: `阶段${index - 2}`
              })
            }}
            editable={{
              type: 'multiple',
              editableKeys,
              onChange: setEditableRowKeys,
              actionRender: (row, _, dom) => {
                return [dom.delete]
              }
            }}
          />
        </ProForm.Item>
      </ModalForm>
    </div>
  )
}

export default Contact
