import React, { useState, useRef, useEffect } from 'react'
import { useRequest, request } from 'umi'
import { Button, Input, message, Modal } from 'antd'
import { fetchStaffList } from '@/services/user/api'
import { LoadingOutlined, MoreOutlined } from '@ant-design/icons'

interface ConnectModalProps {
  postUrl: string
  showButton?: boolean
  title?: string
  dataId?: string
  onReload?: () => void
  show?: boolean
  onShowChange?: (isShow: boolean) => void
}
const ConnectModal: React.FC<ConnectModalProps> = ({
  title,
  dataId,
  onReload,
  show,
  onShowChange,
  postUrl,
  showButton = true
}) => {
  const containerRef = useRef<HTMLDivElement>(null)
  const [visible, setVisible] = useState(false)
  const [searchVal, setSearchVal] = useState('')

  const { run: tryConnectStaff } = useRequest(
    (params: API.AddRoleStaffCommonParams) =>
      request(postUrl, {
        method: 'POST',
        data: params
      }),
    {
      manual: true,
      onSuccess: async () => {
        message.success('关联成功')
        if (!show) {
          setVisible(false)
        }
        await onReload()
      },
      onError: e => {
        message.error(e.message)
      }
    }
  )

  const {
    run: tryQueryStaffList,
    data,
    noMore,
    loadMore,
    loadingMore
  } = useRequest(
    result =>
      fetchStaffList({
        current: result?.list.length / 10 + 1 || 1,
        pageSize: 10,
        search: searchVal
      }),
    {
      manual: true,
      loadMore: true,
      ref: containerRef,
      isNoMore: (res: API.BasicFetchResult<API.StaffItem>) => res.list.length >= res.total,
      refreshDeps: [searchVal]
    }
  )

  const handleSearch = (value: string) => {
    setSearchVal(value)
    setTimeout(() => {
      tryQueryStaffList()
    }, 250)
  }

  const { list = [] }: { list: API.StaffItem[] } = data || {}
  useEffect(() => {
    if (show || visible) {
      tryQueryStaffList()
    }
  }, [show, visible])

  const handleOnCancel = () => {
    if (onShowChange !== undefined) {
      onShowChange(false)
    } else {
      setVisible(false)
    }
  }

  const itemSelectHandler = async (staffId: string) => {
    const params = { staffId }
    if (dataId) {
      params.id = dataId
    }
    await tryConnectStaff(params)
    handleOnCancel()
  }
  return (
    <>
      {showButton && (
        <Button type="primary" onClick={() => setVisible(true)}>
          {title}
        </Button>
      )}
      <Modal
        visible={show === undefined ? visible : show}
        onCancel={handleOnCancel}
        getContainer={false}
        width="60vw"
        title={
          <Input.Search
            placeholder="搜索员工（姓名）"
            onSearch={value => handleSearch(value)}
            onPressEnter={e => handleSearch(e.currentTarget.value)}
            style={{ width: '95%' }}
            allowClear={true}
          />
        }>
        <div ref={containerRef} className="h-60vh overflow-y-auto overflow-x-hidden">
          {list.map(item => (
            <div className="group-item-card" key={item.id}>
              <div className="w-4/3 flex justify-between">
                <span className="w-1/5">{item.username}</span>
                <span className="w-2/5">{item.phone}</span>
                <span className="w-1/5">{item.position}</span>
                <span className="w-1/5">{item.category}</span>
              </div>
              <div className="w-1/4 flex justify-end">
                <span className="btn-outline" onClick={() => itemSelectHandler(item.id)}>
                  选择ta
                </span>
              </div>
            </div>
          ))}
          {noMore && <div className="text-center text-gray-400">已到底部</div>}
          {!noMore && (
            <div className="text-center mt-3 cursor-pointer">
              {loadingMore ? (
                <LoadingOutlined />
              ) : (
                <MoreOutlined rotate={90} style={{ fontSize: 24 }} onClick={loadMore} />
              )}
            </div>
          )}
        </div>
      </Modal>
    </>
  )
}

export default ConnectModal
